@@ -11,6 +11,19 @@ from .attribute import (
     PushButtonParam,
     StringParam,
 )
+from .geometryAttribute import (
+    Geometry,
+    Size2d,
+    Vec2d,
+)
+from .shapeAttribute import (
+    Shape,
+    ShapeList,
+    Point2d,
+    Line2d,
+    Rectangle,
+    Circle
+)
 from .computation import (
     DynamicNodeSize,
     Level,
