@@ -256,7 +256,7 @@ class Graph(BaseObject):
         self._deserialize(Graph._loadGraphData(filepath))
         self._fileDateVersion = os.path.getmtime(filepath)
 
-    def initFromTemplate(self, filepath: PathLike, publishOutputs: bool = False):
+    def initFromTemplate(self, filepath: PathLike, copyOutputs: bool = False):
         """
         Deserialize a template Meshroom Graph ".mg" file in place.
 
@@ -265,7 +265,7 @@ class Graph(BaseObject):
 
         Args:
             filepath: The path to the Meshroom Graph file to load.
-            publishOutputs: (optional) Whether to keep 'Publish' nodes.
+            copyOutputs: (optional) Whether to keep 'CopyFiles' nodes.
         """
         self._deserialize(Graph._loadGraphData(filepath))
 
@@ -274,9 +274,9 @@ class Graph(BaseObject):
         # node instance created by this process.
         self._triggerNodeCreatedCallback(self.nodes)
 
-        if not publishOutputs:
+        if not copyOutputs:
             with GraphModification(self):
-                for node in [node for node in self.nodes if node.nodeType == "Publish"]:
+                for node in [node for node in self.nodes if node.nodeType == "CopyFiles"]:
                     self.removeNode(node.name)
 
     @staticmethod
@@ -560,7 +560,7 @@ class Graph(BaseObject):
 
             # re-create edges taking into account what has been duplicated
             for attr, linkExpression in duplicateEdges.items():
-                # logging.warning("attr={} linkExpression={}".format(attr.fullName, linkExpression))
+                # logging.warning("attr={} linkExpression={}".format(attr.rootName, linkExpression))
                 link = linkExpression[1:-1]  # remove starting '{' and trailing '}'
                 # get source node and attribute name
                 edgeSrcNodeName, edgeSrcAttrName = link.split(".", 1)
@@ -594,12 +594,12 @@ class Graph(BaseObject):
         Remove the node identified by 'nodeName' from the graph.
         Returns:
             - a dictionary containing the incoming edges removed by this operation:
-                {dstAttr.getFullNameToNode(), srcAttr.getFullNameToNode()}
+                {dstAttr.fullName, srcAttr.fullName}
             - a dictionary containing the outgoing edges removed by this operation:
-                {dstAttr.getFullNameToNode(), srcAttr.getFullNameToNode()}
+                {dstAttr.fullName, srcAttr.fullName}
             - a dictionary containing the values, indices and keys of attributes that were connected to a ListAttribute
                 prior to the removal of all edges:
-                {dstAttr.getFullNameToNode(), (dstAttr.root.getFullNameToNode(), dstAttr.index, dstAttr.value)}
+                {dstAttr.fullName, (dstAttr.root.fullName, dstAttr.index, dstAttr.value)}
         """
         node = self.node(nodeName)
         inEdges = {}
@@ -614,13 +614,13 @@ class Graph(BaseObject):
             # - once we have collected all the information, the edges (and perhaps the entries in ListAttributes) can
             #   actually be removed
             for edge in self.nodeOutEdges(node):
-                outEdges[edge.dst.getFullNameToNode()] = edge.src.getFullNameToNode()
+                outEdges[edge.dst.fullName] = edge.src.fullName
 
                 if isinstance(edge.dst.root, ListAttribute):
                     index = edge.dst.root.index(edge.dst)
-                    outListAttributes[edge.dst.getFullNameToNode()] = (edge.dst.root.getFullNameToNode(),
-                                                                       index, edge.dst.value
-                                                                       if edge.dst.value else None)
+                    outListAttributes[edge.dst.fullName] = (edge.dst.root.fullName,
+                                                            index, edge.dst.value
+                                                            if edge.dst.value else None)
 
             for edge in self.nodeOutEdges(node):
                 self.removeEdge(edge.dst)
@@ -632,7 +632,7 @@ class Graph(BaseObject):
 
             for edge in self.nodeInEdges(node):
                 self.removeEdge(edge.dst)
-                inEdges[edge.dst.getFullNameToNode()] = edge.src.getFullNameToNode()
+                inEdges[edge.dst.fullName] = edge.src.fullName
 
             node.alive = False
             self._nodes.remove(node)
@@ -698,12 +698,12 @@ class Graph(BaseObject):
         Returns:
             - the upgraded (newly created) node
             - a dictionary containing the incoming edges removed by this operation:
-                {dstAttr.getFullNameToNode(), srcAttr.getFullNameToNode()}
+                {dstAttr.fullName, srcAttr.fullName}
             - a dictionary containing the outgoing edges removed by this operation:
-                {dstAttr.getFullNameToNode(), srcAttr.getFullNameToNode()}
+                {dstAttr.fullName, srcAttr.fullName}
             - a dictionary containing the values, indices and keys of attributes that were connected to a ListAttribute
                 prior to the removal of all edges:
-                {dstAttr.getFullNameToNode(), (dstAttr.root.getFullNameToNode(), dstAttr.index, dstAttr.value)}
+                {dstAttr.fullName, (dstAttr.root.fullName, dstAttr.index, dstAttr.value)}
         """
         node = self.node(nodeName)
         if not isinstance(node, CompatibilityNode):
@@ -730,10 +730,10 @@ class Graph(BaseObject):
         
         Args:
             outEdges: a dictionary containing the outgoing edges removed by a call to "removeNode".
-                {dstAttr.getFullNameToNode(), srcAttr.getFullNameToNode()}
+                {dstAttr.fullName, srcAttr.fullName}
             outListAttributes: a dictionary containing the values, indices and keys of attributes that were connected
                 to a ListAttribute prior to the removal of all edges.
-                {dstAttr.getFullNameToNode(), (dstAttr.root.getFullNameToNode(), dstAttr.index, dstAttr.value)}
+                {dstAttr.fullName, (dstAttr.root.fullName, dstAttr.index, dstAttr.value)}
         """
         def _recreateTargetListAttributeChildren(listAttrName: str, index: int, value: Any):
             listAttr = self.attribute(listAttrName)
@@ -899,13 +899,13 @@ class Graph(BaseObject):
         if srcAttr.node.graph != self or dstAttr.node.graph != self:
             raise RuntimeError('The attributes of the edge should be part of a common graph.')
         if dstAttr in self.edges.keys():
-            raise RuntimeError(f'Destination attribute "{dstAttr.getFullNameToNode()}" is already connected.')
+            raise RuntimeError(f'Destination attribute "{dstAttr.fullName}" is already connected.')
         edge = Edge(srcAttr, dstAttr)
         self.edges.add(edge)
         self.markNodesDirty(dstAttr.node)
         dstAttr.valueChanged.emit()
-        dstAttr.isLinkChanged.emit()
-        srcAttr.hasOutputConnectionsChanged.emit()
+        dstAttr.inputLinksChanged.emit()
+        srcAttr.outputLinksChanged.emit()
         return edge
 
     def addEdges(self, *edges):
@@ -916,12 +916,12 @@ class Graph(BaseObject):
     @changeTopology
     def removeEdge(self, dstAttr):
         if dstAttr not in self.edges.keys():
-            raise RuntimeError(f'Attribute "{dstAttr.getFullNameToNode()}" is not connected')
+            raise RuntimeError(f'Attribute "{dstAttr.fullName}" is not connected')
         edge = self.edges.pop(dstAttr)
         self.markNodesDirty(dstAttr.node)
         dstAttr.valueChanged.emit()
-        dstAttr.isLinkChanged.emit()
-        edge.src.hasOutputConnectionsChanged.emit()
+        dstAttr.inputLinksChanged.emit()
+        edge.src.outputLinksChanged.emit()
 
     def getDepth(self, node, minimal=False):
         """ Return node's depth in this Graph.
@@ -1242,7 +1242,7 @@ class Graph(BaseObject):
             attr = e.src
             if dependenciesOnly:
                 if attr.isLink:
-                    attr = attr.getLinkParam(recursive=True)
+                    attr = attr.inputRootLink
                 if not attr.isOutput:
                     continue
             newE = Edge(attr, e.dst)
@@ -1711,7 +1711,7 @@ def submitGraph(graph, submitter, toNodes=None, submitLabel="{projectName}"):
         res = sub.submit(nodesToProcess, edgesToProcess, graph.filepath, submitLabel=submitLabel)
         if res:
             for node in nodesToProcess:
-                node.submit()  # update node status
+                node.initStatusOnSubmit()  # update node status
     except Exception as e:
         logging.error(f"Error on submit : {e}")
 
