@@ -1,4 +1,5 @@
 #!/usr/bin/env python
+import sys
 import atexit
 import copy
 import datetime
@@ -9,18 +10,17 @@ import platform
 import re
 import shutil
 import time
-import types
 import uuid
-from collections import namedtuple
+from collections import namedtuple, OrderedDict
 from enum import Enum, auto
-from typing import Callable, Optional
-
+from typing import Callable, Optional, List
 
 import meshroom
 from meshroom.common import Signal, Variant, Property, BaseObject, Slot, ListModel, DictModel
 from meshroom.core import desc, plugins, stats, hashValue, nodeVersion, Version, MrNodeType
 from meshroom.core.attribute import attributeFactory, ListAttribute, GroupAttribute, Attribute
 from meshroom.core.exception import NodeUpgradeError, UnknownNodeTypeError
+from meshroom.core.mtyping import PathLike
 
 
 def getWritingFilepath(filepath: str) -> str:
@@ -220,9 +220,11 @@ class StatusData(BaseObject):
 class LogManager:
     dateTimeFormatting = '%H:%M:%S'
 
-    def __init__(self, chunk):
-        self.chunk = chunk
-        self.logger = logging.getLogger(chunk.node.getName())
+    def __init__(self, logger, logFile):
+        self.logger: logging.Logger = logger
+        self.logFile: PathLike = logFile
+        self._previousHandlers: List[logging.Handler] = []
+        self._previousLevel: int = 0
 
     class Formatter(logging.Formatter):
         def format(self, record):
@@ -231,19 +233,29 @@ class LogManager:
             return logging.Formatter.format(self, record)
 
     def configureLogger(self):
+        self._previousLevel = self.logger.level
+        self._previousHandlers = []
         for handler in self.logger.handlers[:]:
+            self._previousHandlers.append(handler)
             self.logger.removeHandler(handler)
-        handler = logging.FileHandler(self.chunk.logFile)
+        handler = logging.FileHandler(self.logFile)
         formatter = self.Formatter('[%(asctime)s.%(msecs)03d][%(levelname)s] %(message)s',
                                    self.dateTimeFormatting)
         handler.setFormatter(formatter)
         self.logger.addHandler(handler)
 
+    def restorePreviousLogger(self):
+        for h in self.logger.handlers[:]:
+            self.logger.removeHandler(h)
+        for h in self._previousHandlers:
+            self.logger.addHandler(h)
+        self.logger.setLevel(self._previousLevel)
+
     def start(self, level):
         # Clear log file
-        open(self.chunk.logFile, 'w').close()
-
+        open(self.logFile, 'w').close()
         self.configureLogger()
+        self.logger.propagate = False
         self.logger.setLevel(self.textToLevel(level))
         self.progressBar = False
 
@@ -260,7 +272,7 @@ class LogManager:
         self.currentProgressTics = 0
         self.progressBar = True
 
-        with open(self.chunk.logFile, 'a') as f:
+        with open(self.logFile, 'a') as f:
             if message:
                 f.write(message+'\n')
             f.write('0%   10   20   30   40   50   60   70   80   90   100%\n')
@@ -268,7 +280,7 @@ class LogManager:
 
             f.close()
 
-        with open(self.chunk.logFile) as f:
+        with open(self.logFile) as f:
             content = f.read()
             self.progressBarPosition = content.rfind('\n')
 
@@ -280,7 +292,7 @@ class LogManager:
 
         tics = round((value/self.progressEnd)*51)
 
-        with open(self.chunk.logFile, 'r+') as f:
+        with open(self.logFile, 'r+') as f:
             text = f.read()
             for i in range(tics-self.currentProgressTics):
                 text = text[:self.progressBarPosition]+'*'+text[self.progressBarPosition:]
@@ -295,8 +307,10 @@ class LogManager:
 
         self.progressBar = False
 
-    def textToLevel(self, text):
-        if text == "critical":
+    @staticmethod
+    def textToLevel(text):
+        text = text.lower()
+        if text in ["critical", "fatal"]:
             return logging.CRITICAL
         elif text == "error":
             return logging.ERROR
@@ -306,6 +320,8 @@ class LogManager:
             return logging.INFO
         elif text == "debug":
             return logging.DEBUG
+        elif text == "trace":
+            return logging.TRACE
         else:
             return logging.NOTSET
 
@@ -324,7 +340,7 @@ class NodeChunk(BaseObject):
         super().__init__(parent)
         self.node = node
         self.range = range
-        self.logManager: LogManager = LogManager(self)
+        self._logManager = None
         self._status: StatusData = StatusData(node.name, node.nodeType, node.packageName,
                                               node.packageVersion, node.getMrNodeType())
         self.statistics: stats.Statistics = stats.Statistics()
@@ -344,6 +360,13 @@ class NodeChunk(BaseObject):
         else:
             return self.node.name
 
+    @property
+    def logManager(self):
+        if self._logManager is None:
+            logger = logging.getLogger(self.node.getName())
+            self._logManager = LogManager(logger, self.logFile)
+        return self._logManager
+
     @property
     def statusName(self):
         return self._status.status.name
@@ -386,25 +409,24 @@ class NodeChunk(BaseObject):
     @property
     def statusFile(self):
         if self.range.blockSize == 0:
-            return os.path.join(self.node.graph.cacheDir, self.node.internalFolder, "status")
+            return os.path.join(self.node.internalFolder, "status")
         else:
-            return os.path.join(self.node.graph.cacheDir, self.node.internalFolder,
+            return os.path.join(self.node.internalFolder,
                                 str(self.index) + ".status")
 
     @property
     def statisticsFile(self):
         if self.range.blockSize == 0:
-            return os.path.join(self.node.graph.cacheDir, self.node.internalFolder, "statistics")
+            return os.path.join(self.node.internalFolder, "statistics")
         else:
-            return os.path.join(self.node.graph.cacheDir, self.node.internalFolder,
-                                str(self.index) + ".statistics")
+            return os.path.join(self.node.internalFolder, str(self.index) + ".statistics")
 
     @property
     def logFile(self):
         if self.range.blockSize == 0:
-            return os.path.join(self.node.graph.cacheDir, self.node.internalFolder, "log")
+            return os.path.join(self.node.internalFolder, "log")
         else:
-            return os.path.join(self.node.graph.cacheDir, self.node.internalFolder,
+            return os.path.join(self.node.internalFolder,
                                 str(self.index) + ".log")
 
     def saveStatusFile(self):
@@ -498,6 +520,7 @@ class NodeChunk(BaseObject):
         executionStatus = None
         self.statThread = stats.StatisticsThread(self)
         self.statThread.start()
+
         try:
             self.node.nodeDesc.processChunk(self)
             # NOTE: this assumes saving the output attributes for each chunk
@@ -649,7 +672,8 @@ class BaseNode(BaseObject):
         self.packageName: str = ""
         self.packageVersion: str = ""
         self._internalFolder: str = ""
-        self._sourceCodeFolder: str = ""
+        self._sourceCodeFolder: str = self.nodeDesc.sourceCodeFolder if self.nodeDesc else ""
+        self._internalFolderExp = "{cache}/{nodeType}/{uid}"
 
         # temporary unique name for this node
         self._name: str = f"_{nodeType}_{uuid.uuid1()}"
@@ -657,8 +681,9 @@ class BaseNode(BaseObject):
         self.dirty: bool = True  # whether this node's outputs must be re-evaluated on next Graph update
         self._chunks = ListModel(parent=self)
         self._uid: str = uid
-        self._cmdVars: dict = {}
+        self._expVars: dict = {}
         self._size: int = 0
+        self._logManager: Optional[LogManager] = None
         self._position: Position = position or Position()
         self._attributes = DictModel(keyAttrName='name', parent=self)
         self._internalAttributes = DictModel(keyAttrName='name', parent=self)
@@ -670,6 +695,11 @@ class BaseNode(BaseObject):
 
         self.globalStatusChanged.connect(self.updateDuplicatesStatusAndLocked)
 
+        self._staticExpVars = {
+            "nodeType": self.nodeType,
+            "nodeSourceCodeFolder": self.sourceCodeFolder
+        }
+
     def __getattr__(self, k):
         try:
             # Throws exception if not in prototype chain
@@ -704,6 +734,15 @@ class BaseNode(BaseObject):
                 return label
         return self.getDefaultLabel()
 
+    def getNodeLogLevel(self):
+        """
+        Returns:
+            str: the user-provided log level used for logging on process launched by this node
+        """
+        if self.hasInternalAttribute("nodeDefaultLogLevel"):
+            return self.internalAttribute("nodeDefaultLogLevel").value.strip()
+        return "info"
+
     def getColor(self):
         """
         Returns:
@@ -743,8 +782,38 @@ class BaseNode(BaseObject):
     def getDocumentation(self):
         if not self.nodeDesc:
             return ""
-        return self.nodeDesc.documentation
-
+        if self.nodeDesc.documentation:
+            return self.nodeDesc.documentation
+        else:
+            return self.nodeDesc.__doc__
+    
+    def getNodeInfos(self):
+        if not self.nodeDesc:
+            return []
+        infos = OrderedDict([
+            ("module", self.nodeDesc.__module__),
+            ("modulePath", self.nodeDesc.plugin.path),
+        ])
+        # > Infos from the plugin module
+        plugin_module = sys.modules.get(self.nodeDesc.__module__)
+        if getattr(plugin_module, "__author__", None):
+            infos["author"] = plugin_module.__author__
+        if getattr(plugin_module, "__license__", None):
+            infos["license"] = plugin_module.__license__
+        if getattr(plugin_module, "__version__", None):
+            infos["version"] = plugin_module.__version__
+        # > Overrides at the node-level
+        if getattr(self.nodeDesc, "author", None):
+            infos["author"] = self.nodeDesc.author
+        if getattr(self.nodeDesc, "version", None):
+            infos["version"] = self.nodeDesc.version
+        # > Additional node infos stored in a __nodeInfo__ parameter
+        additionalNodeInfos = getattr(self.nodeDesc, "__nodeInfo__", None)
+        if additionalNodeInfos:
+            for key, value in additionalNodeInfos:
+                infos[key] = value
+        return [{"key": k, "value": v} for k, v in infos.items()]
+    
     @property
     def packageFullName(self):
         return '-'.join([self.packageName, self.packageVersion])
@@ -849,7 +918,7 @@ class BaseNode(BaseObject):
 
     @property
     def valuesFile(self):
-        return os.path.join(self.graph.cacheDir, self.internalFolder, 'values')
+        return os.path.join(self.internalFolder, 'values')
 
     def getInputNodes(self, recursive, dependenciesOnly):
         return self.graph.getInputNodes(self, recursive=recursive,
@@ -875,14 +944,14 @@ class BaseNode(BaseObject):
         for attr in self.invalidatingAttributes:
             if not attr.enabled:
                 continue  # Disabled params do not contribute to the uid
-            dynamicOutputAttr = attr.isLink and attr.getLinkParam(recursive=True).desc.isDynamicValue
+            dynamicOutputAttr = attr.isLink and attr.inputRootLink.desc.isDynamicValue
             # For dynamic output attributes, the UID does not depend on the attribute value.
             # In particular, when loading a project file, the UIDs are updated first,
             # and the node status and the dynamic output values are not yet loaded,
             # so we should not read the attribute value.
-            if not dynamicOutputAttr and attr.value == attr.uidIgnoreValue:
+            if not dynamicOutputAttr and not attr.keyable and attr.value == attr.desc.uidIgnoreValue:
                 continue  # For non-dynamic attributes, check if the value should be ignored
-            uidAttributes.append((attr.getName(), attr.uid()))
+            uidAttributes.append((attr.name, attr.uid()))
         uidAttributes.sort()
 
         # Adding the node type prevents ending up with two identical UIDs for different node types
@@ -890,53 +959,48 @@ class BaseNode(BaseObject):
         uidAttributes.append(self.nodeType)
         self._uid = hashValue(uidAttributes)
 
-    def _buildCmdVars(self):
+    def _computeInternalFolder(self, cacheDir):
+        self._internalFolder = self._internalFolderExp.format(
+            cache=cacheDir or self.graph.cacheDir,
+            nodeType=self.nodeType,
+            uid=self._uid)
+
+    def _buildExpVars(self):
         """
         Generate command variables using input attributes and resolved output attributes
         names and values.
         """
-        def _buildAttributeCmdVars(cmdVars, name, attr):
+        def _buildAttributeExpVars(expVars, name, attr):
             if attr.enabled:
-                group = attr.attributeDesc.group(attr.node) \
-                        if isinstance(attr.attributeDesc.group, types.FunctionType) else attr.attributeDesc.group
-                if group is not None:
-                    # If there is a valid command line "group"
-                    v = attr.getValueStr(withQuotes=True)
-                    cmdVars[name] = f"--{name} {v}"
-                    # xxValue is exposed without quotes to allow to compose expressions
-                    cmdVars[name + "Value"] = attr.getValueStr(withQuotes=False)
+                # xxValue is exposed without quotes to allow to compose expressions
+                expVars[name + "Value"] = attr.getValueStr(withQuotes=False)
 
-                    # List elements may give a fully empty string and will not be sent to the command line.
-                    # String attributes will return only quotes if it is empty and thus will be send to the command line.
-                    # But a List of string containing 1 element,
-                    # and this element is an empty string will also return quotes and will be sent to the command line.
-                    if v:
-                        cmdVars[group] = cmdVars.get(group, "") + " " + cmdVars[name]
-                elif isinstance(attr, GroupAttribute):
+                if isinstance(attr, GroupAttribute):
                     assert isinstance(attr.value, DictModel)
                     # If the GroupAttribute is not set in a single command line argument,
                     # the sub-attributes may need to be exposed individually
                     for v in attr._value:
-                        _buildAttributeCmdVars(cmdVars, v.name, v)
+                        _buildAttributeExpVars(expVars, v.name, v)
 
-        self._cmdVars["uid"] = self._uid
-        self._cmdVars["nodeCacheFolder"] = self.internalFolder
-        self._cmdVars["nodeSourceCodeFolder"] = self.sourceCodeFolder
+        self._expVars = {
+            "uid": self._uid,
+            "nodeCacheFolder": self._internalFolder,
+        }
 
         # Evaluate input params
         for name, attr in self._attributes.objects.items():
             if attr.isOutput:
                 continue  # skip outputs
-            _buildAttributeCmdVars(self._cmdVars, name, attr)
+            _buildAttributeExpVars(self._expVars, name, attr)
 
         # For updating output attributes invalidation values
-        cmdVarsNoCache = self._cmdVars.copy()
-        cmdVarsNoCache["cache"] = ""
+        expVarsNoCache = self._expVars.copy()
+        expVarsNoCache["cache"] = ""
 
         # Use "self._internalFolder" instead of "self.internalFolder" because we do not want it to
         # be resolved with the {cache} information ("self.internalFolder" resolves
         # "self._internalFolder")
-        cmdVarsNoCache["nodeCacheFolder"] = self._internalFolder.format(**cmdVarsNoCache)
+        expVarsNoCache["nodeCacheFolder"] = self._internalFolderExp.format(**expVarsNoCache, **self._staticExpVars)
 
         # Evaluate output params
         for name, attr in self._attributes.objects.items():
@@ -944,13 +1008,13 @@ class BaseNode(BaseObject):
                 continue  # skip inputs
 
             # Apply expressions for File attributes
-            if attr.attributeDesc.isExpression:
+            if attr.desc.isExpression:
                 defaultValue = ""
                 # Do not evaluate expression for disabled attributes
                 # (the expression may refer to other attributes that are not defined)
                 if attr.enabled:
                     try:
-                        defaultValue = attr.defaultValue()
+                        defaultValue = attr.getDefaultValue()
                     except AttributeError:
                         # If we load an old scene, the lambda associated to the 'value' could try to
                         # access other params that could not exist yet
@@ -958,8 +1022,8 @@ class BaseNode(BaseObject):
                                         format(nodeName=self.name, attrName=attr.name))
                     if defaultValue is not None:
                         try:
-                            attr.value = defaultValue.format(**self._cmdVars)
-                            attr._invalidationValue = defaultValue.format(**cmdVarsNoCache)
+                            attr.value = defaultValue.format(**self._expVars)
+                            attr._invalidationValue = defaultValue.format(**expVarsNoCache)
                         except KeyError as e:
                             logging.warning('Invalid expression with missing key on "{nodeName}.{attrName}" with '
                                             'value "{defaultValue}".\nError: {err}'.
@@ -971,15 +1035,60 @@ class BaseNode(BaseObject):
                                             format(nodeName=self.name, attrName=attr.name, defaultValue=defaultValue,
                                             err=str(e)))
 
+            # xxValue is exposed without quotes to allow to compose expressions
+            self._expVars[name + 'Value'] = attr.getValueStr(withQuotes=False)
+
+
+    def createCmdLineVars(self):
+        """
+        Generate command variables using input attributes and resolved output attributes
+        names and values.
+        """
+        def _buildAttributeCmdLineVars(cmdLineVars, name, attr):
+            if attr.enabled:
+                group = attr.desc.group(attr.node) \
+                        if callable(attr.desc.group) else attr.desc.group
+                if group:
+                    # If there is a valid command line "group"
+                    v = attr.getValueStr(withQuotes=True)
+
+                    # List elements may give a fully empty string and will not be sent to the command line.
+                    # String attributes will return only quotes if it is empty and thus will be send to the command line.
+                    # But a List of string containing 1 element,
+                    # and this element is an empty string will also return quotes and will be sent to the command line.
+                    if v:
+                        cmdLineVars[group] = cmdLineVars.get(group, "") + f" --{name} {v}"
+                elif isinstance(attr, GroupAttribute):
+                    assert isinstance(attr.value, DictModel)
+                    # If the GroupAttribute is not set in a single command line argument,
+                    # the sub-attributes may need to be exposed individually
+                    for v in attr._value:
+                        _buildAttributeCmdLineVars(cmdLineVars, v.name, v)
+
+        cmdLineVars = {}
+
+        # Evaluate input params
+        for name, attr in self._attributes.objects.items():
+            if attr.isOutput:
+                continue  # skip outputs
+            _buildAttributeCmdLineVars(cmdLineVars, name, attr)
+
+        # Evaluate output params
+        for name, attr in self._attributes.objects.items():
+            if attr.isInput:
+                continue  # skip inputs
+            if not attr.desc.group:
+                continue  # skip attributes without group
+
             v = attr.getValueStr(withQuotes=True)
 
-            self._cmdVars[name] = f'--{name} {v}'
-            # xxValue is exposed without quotes to allow to compose expressions
-            self._cmdVars[name + 'Value'] = attr.getValueStr(withQuotes=False)
+            if not v:
+                continue  # skip empty strings
 
-            if v:
-                self._cmdVars[attr.attributeDesc.group] = \
-                    self._cmdVars.get(attr.attributeDesc.group, '') + ' ' + self._cmdVars[name]
+            cmdLineVars[attr.desc.group] = \
+                cmdLineVars.get(attr.desc.group, '') + f' --{name} {v}'
+
+        return cmdLineVars
 
     @property
     def isParallelized(self):
@@ -1126,6 +1235,9 @@ class BaseNode(BaseObject):
     def _updateChunks(self):
         pass
 
+    def _updateNodeSize(self):
+        self.setSize(self.nodeDesc.size.computeSize(self))
+
     def _getAttributeChangedCallback(self, attr: Attribute) -> Optional[Callable]:
         """ Get the node descriptor-defined value changed callback associated to `attr` if any. """
 
@@ -1158,7 +1270,7 @@ class BaseNode(BaseObject):
             # And we do not want notifications during the graph processing.
             return
 
-        if attr.value is None:
+        if not attr.keyable and attr.value is None:
             # Discard dynamic values depending on the graph processing.
             return
 
@@ -1207,6 +1319,8 @@ class BaseNode(BaseObject):
         for attr in self._attributes:
             attr.updateInternals()
 
+        self._updateNodeSize()
+
         # Update chunks splitting
         self._updateChunks()
         # Retrieve current internal folder (if possible)
@@ -1216,18 +1330,13 @@ class BaseNode(BaseObject):
             folder = ''
 
         # Update command variables / output attributes
-        self._cmdVars = {
-            "cache": cacheDir or self.graph.cacheDir,
-            "nodeType": self.nodeType,
-            "nodeCacheFolder": self._internalFolder,
-            "nodeSourceCodeFolder": self.sourceCodeFolder
-        }
         self._computeUid()
-        self._buildCmdVars()
+        self._computeInternalFolder(cacheDir)
+        self._buildExpVars()
         if self.nodeDesc:
             self.nodeDesc.postUpdate(self)
         # Notify internal folder change if needed
-        if self.internalFolder != folder:
+        if self._internalFolder != folder:
             self.internalFolderChanged.emit()
 
     def updateInternalAttributes(self):
@@ -1235,7 +1344,7 @@ class BaseNode(BaseObject):
 
     @property
     def internalFolder(self):
-        return self._internalFolder.format(**self._cmdVars)
+        return self._internalFolder
 
     @property
     def sourceCodeFolder(self):
@@ -1251,7 +1360,8 @@ class BaseNode(BaseObject):
         # logging.warning(f"updateStatusFromCache: {self.name}, status: {s} => {self.globalStatus}")
         self.updateOutputAttr()
 
-    def submit(self, forceCompute=False):
+    def initStatusOnSubmit(self, forceCompute=False):
+        """ Prepare chunks status when the node is in a graph that was submitted """
         for chunk in self._chunks:
             if forceCompute or chunk.status.status != Status.SUCCESS:
                 chunk._status.setNode(self)
@@ -1281,6 +1391,24 @@ class BaseNode(BaseObject):
         # node is completed
         self.nodeDesc.postprocess(self)
 
+    def getLogHandlers(self):
+        return self._handlers
+
+    def prepareLogger(self, iteration=-1):
+        # Get file handler path
+        logFileName = "log"
+        if iteration != -1:
+            chunk = self.chunks[iteration]
+            logFileName = str(chunk.index) + ".log"
+        logFile = os.path.join(self.internalFolder, logFileName)
+        # Setup logger
+        rootLogger = logging.getLogger()
+        self._logManager = LogManager(rootLogger, logFile)
+        self._logManager.start(self.getNodeLogLevel())
+
+    def restoreLogger(self):
+        self._logManager.restorePreviousLogger()
+
     def updateOutputAttr(self):
         if not self.nodeDesc:
             return
@@ -1414,7 +1542,10 @@ class BaseNode(BaseObject):
 
     @property
     def globalExecMode(self):
-        return self._chunks.at(0).execModeName
+        if len(self._chunks):
+            return self._chunks.at(0).execModeName
+        else:
+            return ExecMode.NONE
 
     def getChunks(self) -> list[NodeChunk]:
         return self._chunks
@@ -1620,12 +1751,23 @@ class BaseNode(BaseObject):
         False otherwise.
         """
         
-        return next((attr for attr in self._attributes if attr.enabled and attr.isOutput and attr.is3D), None) is not None
+        return next((attr for attr in self._attributes if attr.enabled and attr.isOutput and attr.is3dDisplayable), None) is not None
+    
+    def _hasDisplayableShape(self):
+        """
+        Return True if at least one attribute is a ShapeAttribute, a ShapeListAttribute or a shape File.
+        Note: These attributes can be loaded in the the ShapeViewer / ShapeEditor.
+        False otherwise.
+        """
+        return next((attr for attr in self._attributes if attr.hasDisplayableShape or
+                     attr.desc.semantic == "shapeFile"), None) is not None
+    
 
     name = Property(str, getName, constant=True)
     defaultLabel = Property(str, getDefaultLabel, constant=True)
     nodeType = Property(str, nodeType.fget, constant=True)
     documentation = Property(str, getDocumentation, constant=True)
+    nodeInfos = Property(Variant, getNodeInfos, constant=True)
     positionChanged = Signal()
     position = Property(Variant, position.fget, position.fset, notify=positionChanged)
     x = Property(float, lambda self: self._position.x, notify=positionChanged)
@@ -1673,6 +1815,8 @@ class BaseNode(BaseObject):
     hasImageOutput = Property(bool, hasImageOutputAttribute, notify=outputAttrEnabledChanged)
     hasSequenceOutput = Property(bool, hasSequenceOutputAttribute, notify=outputAttrEnabledChanged)
     has3DOutput = Property(bool, has3DOutputAttribute, notify=outputAttrEnabledChanged)
+    # Whether the node contains a ShapeAttribute, a ShapeListAttribute or a shape File.
+    hasDisplayableShape = Property(bool, _hasDisplayableShape, constant=True)
 
 
 class Node(BaseNode):
@@ -1687,8 +1831,6 @@ class Node(BaseNode):
 
         self.packageName = self.nodeDesc.packageName
         self.packageVersion = self.nodeDesc.packageVersion
-        self._internalFolder = "{cache}/{nodeType}/{uid}"
-        self._sourceCodeFolder = self.nodeDesc.sourceCodeFolder
 
         for attrDesc in self.nodeDesc.inputs:
             self._attributes.add(attributeFactory(attrDesc, kwargs.get(attrDesc.name, None),
@@ -1760,9 +1902,9 @@ class Node(BaseNode):
                 pass
 
     def toDict(self):
-        inputs = {k: v.getExportValue() for k, v in self._attributes.objects.items() if v.isInput}
-        internalInputs = {k: v.getExportValue() for k, v in self._internalAttributes.objects.items()}
-        outputs = ({k: v.getExportValue() for k, v in self._attributes.objects.items()
+        inputs = {k: v.getSerializedValue() for k, v in self._attributes.objects.items() if v.isInput}
+        internalInputs = {k: v.getSerializedValue() for k, v in self._internalAttributes.objects.items()}
+        outputs = ({k: v.getSerializedValue() for k, v in self._attributes.objects.items()
                     if v.isOutput and not v.desc.isDynamicValue})
 
         return {
@@ -1774,7 +1916,6 @@ class Node(BaseNode):
                 'split': self.nbParallelizationBlocks
             },
             'uid': self._uid,
-            'internalFolder': self._internalFolder,
             'inputs': {k: v for k, v in inputs.items() if v is not None},  # filter empty values
             'internalInputs': {k: v for k, v in internalInputs.items() if v is not None},
             'outputs': outputs,
@@ -1784,7 +1925,6 @@ class Node(BaseNode):
         """ Update Node's computation task splitting into NodeChunks based on its description """
         if isinstance(self.nodeDesc, desc.InputNode):
             return
-        self.setSize(self.nodeDesc.size.computeSize(self))
         if self.isParallelized:
             try:
                 ranges = self.nodeDesc.parallelization.getRanges(self)
@@ -1838,7 +1978,6 @@ class CompatibilityNode(BaseNode):
         self._inputs = self.nodeDict.get("inputs", {})
         self._internalInputs = self.nodeDict.get("internalInputs", {})
         self.outputs = self.nodeDict.get("outputs", {})
-        self._internalFolder = self.nodeDict.get("internalFolder", "")
         self._uid = self.nodeDict.get("uid", None)
 
         # Restore parallelization settings
@@ -1867,6 +2006,10 @@ class CompatibilityNode(BaseNode):
     def _isCompatibilityNode(self):
         return True
 
+    def _updateNodeSize(self):
+        # Block the recompute of the node size for compatibility nodes
+        pass
+
     @staticmethod
     def attributeDescFromValue(attrName, value, isOutput):
         """
@@ -2012,14 +2155,14 @@ class CompatibilityNode(BaseNode):
         # if node has not been added to a graph, return serialized node inputs
         if not self.graph:
             return self._inputs
-        return {k: v.getExportValue() for k, v in self._attributes.objects.items() if v.isInput}
+        return {k: v.getSerializedValue() for k, v in self._attributes.objects.items() if v.isInput}
 
     @property
     def internalInputs(self):
         """ Get current node's internal attributes """
         if not self.graph:
             return self._internalInputs
-        return {k: v.getExportValue() for k, v in self._internalAttributes.objects.items()}
+        return {k: v.getSerializedValue() for k, v in self._internalAttributes.objects.items()}
 
     def toDict(self):
         """
