@@ -14,7 +14,7 @@ from pathlib import Path
 
 from meshroom.common import BaseObject
 from meshroom.core import desc
-from meshroom.core.desc.node import _MESHROOM_ROOT
+from meshroom.core.desc.node import _MESHROOM_ROOT, _MESHROOM_COMPUTE_DEPS
 
 
 def validateNodeDesc(nodeDesc: desc.Node) -> list:
@@ -97,25 +97,36 @@ class DirTreeProcessEnv(ProcessEnv):
     def __init__(self, folder: str, configEnv: dict[str: str]):
         super().__init__(folder, configEnv, envType=ProcessEnvType.DIRTREE)
 
-        venvLibPaths = glob.glob(f'{folder}/lib*/python[0-9].[0-9]*/site-packages', recursive=False)
+        # If there is a virtual environment, it is expected to be named "venv".
+        # Beside the virtual environment, a standard "bin"/"lib"/"lib64" hierarchy at
+        # the top level of the plugin folder is expected.
+        venvFolder = Path(folder, "venv")
 
-        self.binPaths: list = [str(Path(folder, "bin"))]
-        self.libPaths: list = [str(Path(folder, "lib")), str(Path(folder, "lib64"))]
-        self.pythonPaths: list = [str(Path(folder))] + self.binPaths + venvLibPaths
+        # Find all the libs that are not directly at the "lib*"-level
+        envLibPaths = glob.glob(f'{folder}/lib*/python[0-9].[0-9]*/site-packages',
+                                recursive=False)
+        venvLibPaths = glob.glob(f'{venvFolder}/lib*/python[0-9].[0-9]*/site-packages',
+                                 recursive=False)
+
+        self.binPaths: list = [str(Path(folder, "bin")), str(Path(venvFolder, "bin"))]
+        self.libPaths: list = [str(Path(folder, "lib")), str(Path(folder, "lib64")),
+                               str(Path(venvFolder, "lib")), str(Path(venvFolder, "lib64"))]
+        self.pythonPaths: list = [str(Path(folder)), str(Path(venvFolder))] + \
+                                 self.binPaths + envLibPaths + venvLibPaths
 
         if sys.platform == "win32":
             # For Windows platforms, try and include the content of the virtual env if it exists
             # The virtual env is expected to be named "venv"
-            venvPath = Path(folder, "venv", "Lib", "site-packages")
-            if venvPath.exists():
-                self.pythonPaths.append(venvPath.as_posix())
+            venvLibPath = Path(venvFolder, "Lib", "site-packages")
+            if venvLibPath.exists():
+                self.pythonPaths.append(venvLibPath.as_posix())
         else:
             # For Linux platforms, lib paths may need to be discovered recursively to be properly
             # added to LD_LIBRARY_PATH
             extraLibPaths = []
             regex = re.compile(r"^lib(\d{2})?$")
-            for venvPath in venvLibPaths:
-                for path, directories, _ in os.walk(venvPath):
+            for envPath in envLibPaths + venvLibPaths:
+                for path, directories, _ in os.walk(envPath):
                     for directory in directories:
                         if re.match(regex, directory):
                             extraLibPaths.append(os.path.join(path, directory))
@@ -124,7 +135,7 @@ class DirTreeProcessEnv(ProcessEnv):
         # Setup the environment dictionary
         self._env = os.environ.copy()
         self._env["PYTHONPATH"] = os.pathsep.join(
-            [f"{_MESHROOM_ROOT}"] + self.pythonPaths + [f"{os.getenv('PYTHONPATH', '')}"])
+            [f"{_MESHROOM_ROOT}"] + self.pythonPaths + [os.getenv('PYTHONPATH', '')])
         self._env["LD_LIBRARY_PATH"] = f"{os.pathsep.join(self.libPaths)}{os.pathsep}{os.getenv('LD_LIBRARY_PATH', '')}"
         self._env["PATH"] = f"{os.pathsep.join(self.binPaths)}{os.pathsep}{os.getenv('PATH', '')}"
 
@@ -137,10 +148,12 @@ class DirTreeProcessEnv(ProcessEnv):
             self._env[k] = val
 
 
-
 class RezProcessEnv(ProcessEnv):
     """
     """
+    
+    REZ_DELIMITER_PATTERN = re.compile(r"-|==|>=|>|<=|<")
+    
     def __init__(self, folder: str, configEnv: dict[str: str], uri: str = ""):
         if not uri:
             raise RuntimeError("Missing name of the Rez environment needs to be provided.")
@@ -148,41 +161,58 @@ class RezProcessEnv(ProcessEnv):
 
     def resolveRezSubrequires(self) -> list[str]:
         """
-        Return the list of packages defined for the node execution. These execution packages are named
-        subrequires.
-        Note: If a package does not have a version number, the version is aligned with the main Meshroom
-        environment (if this package is defined).
+        Return the list of packages defined for the node execution. These execution packages are
+        named subrequires.
+        Note: If a package does not have a version number, the version is aligned with the main
+        Meshroom environment (if this package is defined).
         """
         subrequires = os.environ.get(f"{self.uri.upper()}_SUBREQUIRES", "").split(os.pathsep)
-        packages = []
+        if not subrequires:
+            return []
 
+        packages = []
         # Packages that are resolved in the current environment
         currentEnvPackages = []
-        if "REZ_REQUEST" in os.environ:
-            resolvedPackages = os.getenv("REZ_RESOLVE", "").split()
-            resolvedVersions = {}
+        resolvedVersions = {}
+        if "REZ_USED_RESOLVE" in os.environ:
+            resolvedPackages = os.getenv("REZ_USED_RESOLVE", "").split()
             for package in resolvedPackages:
                 if package.startswith("~"):
                     continue
-                version = package.split("-")
-                resolvedVersions[version[0]] = version[1]
-            currentEnvPackages = [package + "-" + resolvedVersions[package] for package in resolvedVersions.keys()]
+                currentEnvPackages.append(package)
+                name, version = self.REZ_DELIMITER_PATTERN.split(package, maxsplit=1)
+                resolvedVersions[name] = version
         logging.debug("Packages in the current environment: " + ", ".join(currentEnvPackages))
 
-        # Take packages with the set versions for those which have one, and try to take packages in the current
-        # environment (if they are resolved in it)
+        # Take packages with the set versions for those which have one, and try to take packages
+        # in the current environment (if they are resolved in it)
         for package in subrequires:
-            if "-" in package:
-                packages.append(package)
-            else:
-                definedInParentEnv = False
-                for p in currentEnvPackages:
-                    if p.startswith(package + "-"):
-                        packages.append(p)
-                        definedInParentEnv = True
-                        break
-                if not definedInParentEnv:
+            packageTuple = self.REZ_DELIMITER_PATTERN.split(package, maxsplit=1)
+            if len(packageTuple) == 1:
+                # Only the package name in the subrequires.
+                # Search for a corresponding version in the parent environment.
+                packageName = packageTuple[0]
+                parentResolvedVersion = resolvedVersions.get(packageName)
+                if parentResolvedVersion:
+                    packages.append(f"{packageName}=={parentResolvedVersion}")
+                else:
                     packages.append(package)
+            elif len(packageTuple) == 2:
+                # The subrequires ask for a specific version
+                packages.append(package)
+
+        def extractPackageName(packageString: str) -> str:
+            return self.REZ_DELIMITER_PATTERN.split(packageString, maxsplit=1)[0]
+        packageNames = [extractPackageName(package) for package in packages]
+
+        for package in _MESHROOM_COMPUTE_DEPS:
+            # For packages that are required by meshroom_compute, do not specify any version
+            # or align it with Meshroom's: the version will be found during the resolution of
+            # the environment based on the other packages.
+            # If any of these packages is already part of the environment a plugin's dependency,
+            # do not add it
+            if package not in packageNames:
+                packages.append(package)
 
         logging.debug("Packages for the execution environment: " + ", ".join(packages))
         return packages
