@@ -3,10 +3,11 @@ import os
 import re
 import argparse
 import json
+from enum import Enum
 
 from PySide6 import __version__ as PySideVersion
 from PySide6 import QtCore
-from PySide6.QtCore import QUrl, QJsonValue, qInstallMessageHandler, QtMsgType, QSettings
+from PySide6.QtCore import QObject, QUrl, QJsonValue, qInstallMessageHandler, QtMsgType, QSettings
 from PySide6.QtGui import QIcon
 from PySide6.QtQml import QQmlDebuggingEnabler
 from PySide6.QtQuickControls2 import QQuickStyle
@@ -14,6 +15,7 @@ from PySide6.QtWidgets import QApplication
 
 import meshroom
 from meshroom.core import pluginManager
+from meshroom.core.submitter import BaseSubmitter
 from meshroom.core.taskManager import TaskManager
 from meshroom.common import Property, Variant, Signal, Slot
 
@@ -25,12 +27,20 @@ from meshroom.ui.components.filepath import FilepathHelper
 from meshroom.ui.components.scene3D import Scene3DHelper, Transformations3DHelper
 from meshroom.ui.components.scriptEditor import ScriptEditorManager
 from meshroom.ui.components.thumbnail import ThumbnailCache
+from meshroom.ui.components.messaging import MessageController
+from meshroom.ui.components.shapes import ShapeFilesHelper, ShapeViewerHelper
 from meshroom.ui.palette import PaletteManager
 from meshroom.ui.reconstruction import Reconstruction
 from meshroom.ui.utils import QmlInstantEngine
 from meshroom.ui import commands
 
 
+class FileStatus(Enum):
+    MISSING=0
+    EXISTS=1
+    ERROR=2  # If the file exists but have errors like missing nodes, file content corruption...
+
+
 class MessageHandler:
     """
     MessageHandler that translates Qt logs to Python logging system.
@@ -265,6 +275,7 @@ class MeshroomApp(QApplication):
 
         # instantiate Reconstruction object
         self._undoStack = commands.UndoStack(self)
+        self._defaultSubmitterName = os.environ.get('MESHROOM_DEFAULT_SUBMITTER', '')
         self._taskManager = TaskManager(self)
         self._activeProject = Reconstruction(undoStack=self._undoStack, taskManager=self._taskManager, defaultPipeline=args.pipeline, parent=self)
         self._activeProject.setSubmitLabel(args.submitLabel)
@@ -279,8 +290,12 @@ class MeshroomApp(QApplication):
         self.engine.rootContext().setContextProperty("Transformations3DHelper", Transformations3DHelper(parent=self))
         self.engine.rootContext().setContextProperty("Clipboard", ClipboardHelper(parent=self))
         self.engine.rootContext().setContextProperty("ThumbnailCache", ThumbnailCache(parent=self))
+        self.engine.rootContext().setContextProperty("ShapeFilesHelper", ShapeFilesHelper(self.activeProject, parent=self))
+        self.engine.rootContext().setContextProperty("ShapeViewerHelper", ShapeViewerHelper(parent=self))
 
         # additional context properties
+        self._messageController = MessageController(parent=self)
+        self.engine.rootContext().setContextProperty("_messageController", self._messageController)
         self.engine.rootContext().setContextProperty("_PaletteManager", PaletteManager(self.engine, parent=self))
         self.engine.rootContext().setContextProperty("ScriptEditorManager", ScriptEditorManager(parent=self))
         self.engine.rootContext().setContextProperty("MeshroomApp", self)
@@ -353,6 +368,16 @@ class MeshroomApp(QApplication):
     def reloadTemplateList(self):
         meshroom.core.initPipelines()
         self.pipelineTemplateFilesChanged.emit()
+    
+    @Slot()
+    def forceUIUpdate(self):
+        """ Force UI to process pending events
+        Necessary when we want to update the UI while a trigger is still running (e.g. reloadPlugins)
+        """
+        self.processEvents()
+    
+    def showMessage(self, message, status=None, duration=5000):
+        self._messageController.sendMessage(message, status, duration)
 
     def _retrieveThumbnailPath(self, filepath: str) -> str:
         """
@@ -404,7 +429,8 @@ class MeshroomApp(QApplication):
             settings.setArrayIndex(i)
             path = settings.value("filepath")
             if path:
-                p = {"path": path, "thumbnail": self._retrieveThumbnailPath(path)}
+                fileStatus = FileStatus.EXISTS if os.path.isfile(path) else FileStatus.MISSING
+                p = {"path": path, "thumbnail": self._retrieveThumbnailPath(path), "status": fileStatus.value}
                 projects.append(p)
         settings.endArray()
         settings.endGroup()
@@ -425,6 +451,7 @@ class MeshroomApp(QApplication):
         for project in self._recentProjectFiles:
             path = project["path"]
             project["thumbnail"] = self._retrieveThumbnailPath(path)
+            project["status"] = os.path.isfile(path)
 
     @Slot(str)
     @Slot(QUrl)
@@ -461,7 +488,7 @@ class MeshroomApp(QApplication):
             del projects[idx]  # If so, delete its entry
 
         # Insert the newest entry at the top of the list
-        projects.insert(0, {"path": projectFileNorm, "thumbnail": ""})
+        projects.insert(0, {"path": projectFileNorm, "thumbnail": "", "status": FileStatus.EXISTS})
 
         # Only keep the first 40 projects
         maxNbProjects = 40
@@ -698,7 +725,32 @@ class MeshroomApp(QApplication):
         if val != True and str(val).lower() in ("0", "false", "off"):
             return False
         return True
-
+    
+    def _submittersList(self):
+        """
+        Get the list of available submitters
+        Model provides :
+            name : the name of the submitter
+            isDefault : True if this is the current submitter
+        """
+        submittersList = []
+        for i, s in enumerate(meshroom.core.submitters):
+            submitterName = s.name if isinstance(s, BaseSubmitter) else s
+            # If no explicit default submitter, this will be the first one
+            isDefault = (i == 0)
+            if self._defaultSubmitterName:
+                isDefault = (submitterName == self._defaultSubmitterName)
+            submittersList.append({
+                "name": submitterName,
+                "isDefault": isDefault
+            })
+        return submittersList
+    
+    @Slot(str)
+    def setDefaultSubmitter(self, name):
+        logging.info(f"Submitter is now set to : {name}")
+        self._defaultSubmitterName = name
+    
     activeProjectChanged = Signal()
     activeProject = Property(Variant, lambda self: self._activeProject, notify=activeProjectChanged)
 
@@ -713,3 +765,4 @@ class MeshroomApp(QApplication):
     recentImportedImagesFolders = Property("QVariantList", _recentImportedImagesFolders, notify=recentImportedImagesFoldersChanged)
     default8bitViewerEnabled = Property(bool, _default8bitViewerEnabled, constant=True)
     defaultSequencePlayerEnabled = Property(bool, _defaultSequencePlayerEnabled, constant=True)
+    submittersListModel = Property("QVariantList", _submittersList, constant=True)
