@@ -286,10 +286,10 @@ class ImportProjectCommand(GraphCommand):
 class SetAttributeCommand(GraphCommand):
     def __init__(self, graph, attribute, value, parent=None):
         super().__init__(graph, parent)
-        self.attrName = attribute.getFullNameToNode()
+        self.attrName = attribute.fullName
         self.value = value
-        self.oldValue = attribute.getExportValue()
-        self.setText(f"Set Attribute '{attribute.getFullNameToNode()}'")
+        self.oldValue = attribute.getSerializedValue()
+        self.setText(f"Set Attribute '{attribute.fullName}'")
 
     def redoImpl(self):
         if self.value == self.oldValue:
@@ -305,13 +305,128 @@ class SetAttributeCommand(GraphCommand):
             self.graph.attribute(self.attrName).value = self.oldValue
         else:
             self.graph.internalAttribute(self.attrName).value = self.oldValue
+    
+class AddAttributeKeyValueCommand(GraphCommand):
+    def __init__(self, graph, attribute, key, value, parent=None):
+        super().__init__(graph, parent)
+        self.attrName = attribute.fullName
+        self.keyable = attribute.keyable
+        self.key = key
+        self.value = value
+        self.oldValue = None
+        if attribute.keyable and attribute.keyValues.hasKey(key):
+             self.oldValue = attribute.keyValues.pairs.get(int(key)).value
+        self.setText(f"Add (key, value) for attribute '{attribute.fullName}' at key: '{key}'")
 
+    def redoImpl(self):
+        if not self.keyable or self.value == self.oldValue:
+            return False
+        if self.graph.attribute(self.attrName) is not None:
+            self.graph.attribute(self.attrName).keyValues.add(self.key, self.value)
+        else:
+            self.graph.internalAttribute(self.attrName).keyValues.add(self.key, self.value)
+        return True
+
+    def undoImpl(self):
+        if not self.keyable or self.value == self.oldValue:
+            return False    
+        if self.graph.attribute(self.attrName) is not None:
+            if self.oldValue is None:
+                self.graph.attribute(self.attrName).keyValues.remove(self.key)
+            else:
+                self.graph.attribute(self.attrName).keyValues.add(self.key, self.oldValue)
+        else:
+            if self.oldValue is None:
+                self.graph.internalAttribute(self.attrName).keyValues.remove(self.key)
+            else:
+                self.graph.internalAttribute(self.attrName).keyValues.add(self.key, self.oldValue)
+        return True
+
+class RemoveAttributeKeyCommand(GraphCommand):
+    def __init__(self, graph, attribute, key, parent=None):
+        super().__init__(graph, parent)
+        self.attrName = attribute.fullName
+        self.keyable = attribute.keyable
+        self.key = key
+        self.oldValue = None
+        if attribute.keyable and attribute.keyValues.hasKey(key):
+             self.oldValue = attribute.keyValues.pairs.get(int(key)).value
+        self.setText(f"Remove (key, value) for attribute '{attribute.fullName}' at key: '{key}'")
+
+    def redoImpl(self):
+        if not self.keyable or self.oldValue == None:
+            return False
+        if self.graph.attribute(self.attrName) is not None:
+            self.graph.attribute(self.attrName).keyValues.remove(self.key)
+        else:
+            self.graph.internalAttribute(self.attrName).keyValues.remove(self.key)
+        return True
+
+    def undoImpl(self):
+        if not self.keyable or self.oldValue == None:
+            return False    
+        if self.graph.attribute(self.attrName) is not None:
+            self.graph.attribute(self.attrName).keyValues.add(self.key, self.oldValue)
+        else:
+            self.graph.internalAttribute(self.attrName).keyValues.add(self.key, self.oldValue)
+        return True
+
+class SetObservationCommand(GraphCommand):
+    def __init__(self, graph, attribute, key, observation, parent=None):
+        super().__init__(graph, parent)
+        self.attrName = attribute.fullName
+        self.key = key
+        self.observation = observation.toVariant()
+        self.oldObservation = attribute.geometry.getObservation(key)
+        self.setText(f"Set observation for shape attribute '{attribute.fullName}' at key: '{key}'")
+
+    def redoImpl(self):
+        if self.graph.attribute(self.attrName) is not None:
+            self.graph.attribute(self.attrName).geometry.setObservation(self.key, self.observation)
+        else:
+            self.graph.internalAttribute(self.attrName).geometry.setObservation(self.key, self.observation)
+        return True
+
+    def undoImpl(self):
+        if self.graph.attribute(self.attrName) is not None:
+            if self.oldObservation is None:
+                self.graph.attribute(self.attrName).geometry.removeObservation(self.key)
+            else:
+                self.graph.attribute(self.attrName).geometry.setObservation(self.key, self.oldObservation)
+        else:
+            if self.oldObservation is None:
+                self.graph.internalAttribute(self.attrName).geometry.removeObservation(self.key)
+            else:
+                self.graph.internalAttribute(self.attrName).geometry.setObservation(self.key, self.oldObservation)
+        return True
+
+class RemoveObservationCommand(GraphCommand):
+    def __init__(self, graph, attribute, key, parent=None):
+        super().__init__(graph, parent)
+        self.attrName = attribute.fullName
+        self.key = key
+        self.oldObservation = attribute.geometry.getObservation(key)
+        self.setText(f"Remove observation for shape attribute '{attribute.fullName}' at key: '{key}'")
+
+    def redoImpl(self):
+        if self.graph.attribute(self.attrName) is not None:
+            self.graph.attribute(self.attrName).geometry.removeObservation(self.key)
+        else:
+            self.graph.internalAttribute(self.attrName).geometry.removeObservation(self.key)
+        return True
+
+    def undoImpl(self):
+        if self.graph.attribute(self.attrName) is not None:
+            self.graph.attribute(self.attrName).geometry.setObservation(self.key, self.oldObservation)
+        else:
+            self.graph.internalAttribute(self.attrName).geometry.setObservation(self.key, self.oldObservation)
+        return True
 
 class AddEdgeCommand(GraphCommand):
     def __init__(self, graph, src, dst, parent=None):
         super().__init__(graph, parent)
-        self.srcAttr = src.getFullNameToNode()
-        self.dstAttr = dst.getFullNameToNode()
+        self.srcAttr = src.fullName
+        self.dstAttr = dst.fullName
         self.setText(f"Connect '{self.srcAttr}'->'{self.dstAttr}'")
 
         if src.baseType != dst.baseType:
@@ -328,8 +443,8 @@ class AddEdgeCommand(GraphCommand):
 class RemoveEdgeCommand(GraphCommand):
     def __init__(self, graph, edge, parent=None):
         super().__init__(graph, parent)
-        self.srcAttr = edge.src.getFullNameToNode()
-        self.dstAttr = edge.dst.getFullNameToNode()
+        self.srcAttr = edge.src.fullName
+        self.dstAttr = edge.dst.fullName
         self.setText(f"Disconnect '{self.srcAttr}'->'{self.dstAttr}'")
 
     def redoImpl(self):
@@ -345,7 +460,7 @@ class ListAttributeAppendCommand(GraphCommand):
     def __init__(self, graph, listAttribute, value, parent=None):
         super().__init__(graph, parent)
         assert isinstance(listAttribute, ListAttribute)
-        self.attrName = listAttribute.getFullNameToNode()
+        self.attrName = listAttribute.fullName
         self.index = None
         self.count = 1
         self.value = value if value else None
@@ -371,10 +486,10 @@ class ListAttributeRemoveCommand(GraphCommand):
         super().__init__(graph, parent)
         listAttribute = attribute.root
         assert isinstance(listAttribute, ListAttribute)
-        self.listAttrName = listAttribute.getFullNameToNode()
+        self.listAttrName = listAttribute.fullName
         self.index = listAttribute.index(attribute)
-        self.value = attribute.getExportValue()
-        self.setText(f"Remove {attribute.getFullNameToNode()}")
+        self.value = attribute.getSerializedValue()
+        self.setText(f"Remove {attribute.fullName}")
 
     def redoImpl(self):
         listAttribute = self.graph.attribute(self.listAttrName)
@@ -390,8 +505,8 @@ class RemoveImagesCommand(GraphCommand):
     def __init__(self, graph, cameraInitNodes, parent=None):
         super().__init__(graph, parent)
         self.cameraInits = cameraInitNodes
-        self.viewpoints = { cameraInit.name: cameraInit.attribute("viewpoints").getExportValue() for cameraInit in self.cameraInits }
-        self.intrinsics = { cameraInit.name: cameraInit.attribute("intrinsics").getExportValue() for cameraInit in self.cameraInits }
+        self.viewpoints = { cameraInit.name: cameraInit.attribute("viewpoints").getSerializedValue() for cameraInit in self.cameraInits }
+        self.intrinsics = { cameraInit.name: cameraInit.attribute("intrinsics").getSerializedValue() for cameraInit in self.cameraInits }
         self.title = f"Remove{' ' if len(self.cameraInits) == 1 else ' All '}Images"
         self.setText(self.title)
 
