@@ -23,7 +23,7 @@ from PySide6.QtCore import (
 
 from meshroom.core import sessionUid
 from meshroom.common.qt import QObjectListModel
-from meshroom.core.attribute import Attribute, ListAttribute
+from meshroom.core.attribute import Attribute, ListAttribute, ShapeAttribute
 from meshroom.core.graph import Graph, Edge, generateTempProjectFilepath
 from meshroom.core.graphIO import GraphIO
 
@@ -469,10 +469,10 @@ class UIGraph(QObject):
         self.setGraph(g)
 
     @Slot(str, bool, result=bool)
-    def initFromTemplate(self, filepath, publishOutputs=False):
+    def initFromTemplate(self, filepath, copyOutputs=False):
         graph = Graph("")
         if filepath:
-            graph.initFromTemplate(filepath, publishOutputs=publishOutputs)
+            graph.initFromTemplate(filepath, copyOutputs=copyOutputs)
         self.setGraph(graph)
 
     @Slot(QUrl, result="QVariantList")
@@ -585,7 +585,9 @@ class UIGraph(QObject):
         self.save()  # graph must be saved before being submitted
         self._undoStack.clear()  # the undo stack must be cleared
         nodes = [nodes] if not isinstance(nodes, Iterable) and nodes else nodes
-        self._taskManager.submit(self._graph, os.environ.get('MESHROOM_DEFAULT_SUBMITTER', ''), nodes, submitLabel=self.submitLabel)
+        mrDefaultSubmitter = os.environ.get('MESHROOM_DEFAULT_SUBMITTER', '')
+        chosenSubmitter = self.parent()._defaultSubmitterName or mrDefaultSubmitter
+        self._taskManager.submit(self._graph, chosenSubmitter, nodes, submitLabel=self.submitLabel)
 
     def updateGraphComputingStatus(self):
         # update graph computing status
@@ -690,6 +692,51 @@ class UIGraph(QObject):
                 position = Position(node.x + offset.x(), node.y + offset.y())
                 self.moveNode(node, position)
 
+    def getMeanPosition(self):
+        """ Get the average Position of selected nodes """
+        # Not great, would be better if Position was a non-tuple class
+        selectedNodes = self.getSelectedNodes()
+        sum_pose = [0, 0]
+        nb_tot = 0
+        for selectedNode in selectedNodes:
+            sum_pose[0] += selectedNode.x
+            sum_pose[1] += selectedNode.y
+            nb_tot += 1
+        return Position(int(sum_pose[0] / nb_tot), int(sum_pose[1] / nb_tot))
+
+    @Slot()
+    def alignHorizontally(self):
+        """ All nodes are moved horizontally to the same position, on an average position of selected nodes """
+        nodePadding = 50
+        selectedNodes = self.getSelectedNodes()
+        if len(selectedNodes) < 2:
+            return
+        
+        # Make sure the list is correctly ordered
+        selectedNodes = sorted(selectedNodes, key=lambda node:node.x)
+        
+        meanX, meanY = self.getMeanPosition()
+        nodeWidth = self.layout.nodeWidth
+        # Compute the first node X position
+        totalWidth = len(selectedNodes) * nodeWidth + (len(selectedNodes) - 1) * nodePadding
+        startX = int(meanX - totalWidth / 2 + nodeWidth / 2)
+        with self.groupedGraphModification("Align nodes horizontally"):
+            for i, selectedNode in enumerate(selectedNodes):
+                x = startX + i * (nodeWidth + nodePadding)
+                self.moveNode(selectedNode, Position(x, meanY))
+
+    @Slot()
+    def alignVertically(self):
+        """ All nodes are moved vertically to the same position, on an average position of selected nodes """
+        selectedNodes = self.getSelectedNodes()
+        if len(selectedNodes) < 2:
+            return
+        
+        meanX, _ = self.getMeanPosition()
+        with self.groupedGraphModification("Align nodes vertically"):
+            for selectedNode in selectedNodes:
+                self.moveNode(selectedNode, Position(meanX, selectedNode.y))
+
     @Slot()
     def removeSelectedNodes(self):
         """Remove selected nodes from the graph."""
@@ -849,14 +896,14 @@ class UIGraph(QObject):
         if isinstance(src, ListAttribute) and not isinstance(dst, ListAttribute):
             self._addEdge(src.at(0), dst)
         elif isinstance(dst, ListAttribute) and not isinstance(src, ListAttribute):
-            with self.groupedGraphModification(f"Insert and Add Edge on {dst.getFullNameToNode()}"):
+            with self.groupedGraphModification(f"Insert and Add Edge on {dst.fullName}"):
                 self.appendAttribute(dst)
                 self._addEdge(src, dst.at(-1))
         else:
             self._addEdge(src, dst)
 
     def _addEdge(self, src, dst):
-        with self.groupedGraphModification(f"Connect '{src.getFullNameToNode()}'->'{dst.getFullNameToNode()}'"):
+        with self.groupedGraphModification(f"Connect '{src.fullName}'->'{dst.fullName}'"):
             if dst in self._graph.edges.keys():
                 self.removeEdge(self._graph.edge(dst))
             self.push(commands.AddEdgeCommand(self._graph, src, dst))
@@ -864,7 +911,7 @@ class UIGraph(QObject):
     @Slot(Edge)
     def removeEdge(self, edge):
         if isinstance(edge.dst.root, ListAttribute):
-            with self.groupedGraphModification(f"Remove Edge and Delete {edge.dst.getFullNameToNode()}"):
+            with self.groupedGraphModification(f"Remove Edge and Delete {edge.dst.fullName}"):
                 self.push(commands.RemoveEdgeCommand(self._graph, edge))
                 self.removeAttribute(edge.dst)
         else:
@@ -891,7 +938,7 @@ class UIGraph(QObject):
 
     @Slot(Edge, Attribute, Attribute, result=Edge)
     def replaceEdge(self, edge, newSrc, newDst):
-        with self.groupedGraphModification(f"Replace Edge '{edge.src.getFullNameToNode()}'->'{edge.dst.getFullNameToNode()}' with '{newSrc.getFullNameToNode()}'->'{newDst.getFullNameToNode()}'"):
+        with self.groupedGraphModification(f"Replace Edge '{edge.src.fullName}'->'{edge.dst.fullName}' with '{newSrc.fullName}'->'{newDst.fullName}'"):
             self.removeEdge(edge)
             self.addEdge(newSrc, newDst)
         return self._graph.edge(newDst)
@@ -914,7 +961,40 @@ class UIGraph(QObject):
                     # if the edge is connected to one of the ListAttribute's elements, remove it
                     if edge.src in attribute.value:
                         self.removeEdge(edge)
-            self.push(commands.SetAttributeCommand(self._graph, attribute, attribute.defaultValue()))
+            self.push(commands.SetAttributeCommand(self._graph, attribute, attribute.getDefaultValue()))
+
+    @Slot(Attribute, str, "QVariant")
+    def addAttributeKeyValue(self, attribute, key, value):
+        """ Add the given (key, value) pair to the given keyable attribute. """
+        self.push(commands.AddAttributeKeyValueCommand(self._graph, attribute, key, value))
+
+    @Slot(Attribute, str)
+    def addAttributeKeyDefaultValue(self, attribute, key):
+        """ Add the given key with the default value to the given keyable attribute. """
+        self.push(commands.AddAttributeKeyValueCommand(self._graph, attribute, key, attribute.getDefaultValue()))
+
+    @Slot(Attribute, str)
+    def removeAttributeKey(self, attribute, key):
+        """ Remove the given key from the given keyable attribute. """
+        self.push(commands.RemoveAttributeKeyCommand(self._graph, attribute, key))
+
+    @Slot(str, str, "QVariant")
+    def setObservationFromName(self, shapeFullName, key, observation):
+        """ Set the given observation for the given shape attribute name. """
+        shape = self.graph.attribute(shapeFullName)
+        if shape is None:
+            shape = self.graph.internalAttribute(shapeFullName)
+        self.push(commands.SetObservationCommand(self._graph, shape, key, observation))
+
+    @Slot(ShapeAttribute, str, "QVariant")
+    def setObservation(self, shape, key, observation):
+        """ Set the given observation for the given shape attribute. """
+        self.push(commands.SetObservationCommand(self._graph, shape, key, observation))
+
+    @Slot(ShapeAttribute, str)
+    def removeObservation(self, shape, key):
+        """ Remove the given observation for the given shape attribute. """
+        self.push(commands.RemoveObservationCommand(self._graph, shape, key))
 
     @Slot(CompatibilityNode, result=Node)
     def upgradeNode(self, node):
@@ -960,15 +1040,15 @@ class UIGraph(QObject):
             intrinsicId = image.intrinsicId.value
 
             intrinsicUsed = False
-            for intrinsic in self.cameraInit.attribute("viewpoints").getExportValue():
-                if image.getExportValue() != intrinsic and intrinsic['intrinsicId'] == intrinsicId:
+            for intrinsic in self.cameraInit.attribute("viewpoints").getSerializedValue():
+                if image.getSerializedValue() != intrinsic and intrinsic['intrinsicId'] == intrinsicId:
                     intrinsicUsed = True
                     break
 
             if not intrinsicUsed:
                 #find the intrinsic and remove it
                 for intrinsic in self.cameraInit.attribute("intrinsics"):
-                    if intrinsic.getExportValue()["intrinsicId"] == intrinsicId:
+                    if intrinsic.getSerializedValue()["intrinsicId"] == intrinsicId:
                         self.removeAttribute(intrinsic)
                         break
 
@@ -1122,8 +1202,29 @@ class UIGraph(QObject):
             logging.warning("Content is not a valid graph data.")
             return []
         return result
-
-
+    
+    @Slot(Node, result=bool)
+    def canComputeNode(self, node: Node) -> bool:
+        """ Check if the node can be computed """
+        if node.isCompatibilityNode or not node.isComputableType or node.getLocked():
+            return False
+        if node.isComputed:
+            return True
+        if self._graph.canComputeTopologically(node) and self._graph.canSubmitOrCompute(node) % 2 == 1:
+            return True
+        return False
+    
+    @Slot(Node, result=bool)
+    def canSubmitNode(self, node: Node) -> bool:
+        """ Check if the node can be submitted """
+        if node.isCompatibilityNode or not node.isComputableType or node.getLocked():
+            return False
+        if node.isComputed:
+            return True
+        if self._graph.canComputeTopologically(node) and self._graph.canSubmitOrCompute(node)> 1:
+            return True
+        return False
+    
     undoStack = Property(QObject, lambda self: self._undoStack, constant=True)
     graphChanged = Signal()
     graph = Property(Graph, lambda self: self._graph, notify=graphChanged)
