@@ -18,6 +18,7 @@ from meshroom.common.qt import QObjectListModel
 from meshroom.core import Version
 from meshroom.core.node import Node, CompatibilityNode, Status, Position, CompatibilityIssue
 from meshroom.core.taskManager import TaskManager
+from meshroom.core.evaluation import MathEvaluator
 
 from meshroom.ui import commands
 from meshroom.ui.graph import UIGraph
@@ -329,9 +330,9 @@ class Reconstruction(UIGraph):
         # All nodes generating depth map files
         "allDepthMap": ["DepthMap", "DepthMapFilter"],
         # Nodes that can be used to provide features folders to the UI
-        "featureProvider": ["FeatureExtraction", "FeatureMatching", "StructureFromMotion"],
+        "featureProvider": ["FeatureExtraction", "FeatureMatching", "StructureFromMotion", "RomaReducer"],
         # Nodes that can be used to provide matches folders to the UI
-        "matchProvider": ["FeatureMatching", "StructureFromMotion"],
+        "matchProvider": ["FeatureMatching", "StructureFromMotion", "RomaReducer"],
         # Nodes that can be used to provide tracks files to the UI
         "trackProvider": ["TracksBuilding", "SfMBootstraping", "SfMExpanding"]
     }
@@ -386,6 +387,9 @@ class Reconstruction(UIGraph):
         # react to internal graph changes to update those variables
         self.graphChanged.connect(self.onGraphChanged)
 
+        # Connect the pluginsReloaded signal to the onPluginsReloaded function
+        self.pluginsReloaded.connect(self._onPluginsReloaded)
+
         self.setDefaultPipeline(defaultPipeline)
 
     def __del__(self):
@@ -432,6 +436,10 @@ class Reconstruction(UIGraph):
 
     @Slot()
     def reloadPlugins(self):
+        """ Launch _reloadPlugins in a worker thread to avoid blocking the ui. """
+        self._workerThreads.apply_async(func=self._reloadPlugins, args=())
+
+    def _reloadPlugins(self):
         """
         Reload all the NodePlugins from all the registered plugins.
         The nodes in the graph will be updated to match the changes in the description, if
@@ -442,8 +450,12 @@ class Reconstruction(UIGraph):
             for node in plugin.nodes.values():
                 if node.reload():
                     nodeTypes.append(node.nodeDescriptor.__name__)
+        self.pluginsReloaded.emit(nodeTypes)
 
+    @Slot(list)
+    def _onPluginsReloaded(self, nodeTypes: list):
         self._graph.reloadNodePlugins(nodeTypes)
+        self.parent().showMessage("Plugins reloaded!", "ok")
 
     @Slot()
     @Slot(str)
@@ -828,7 +840,7 @@ class Reconstruction(UIGraph):
         self.error.emit(
             Message(
                 "Failed to Import Images",
-                "You probably have a corrupted image within the images that you are trying to import.",
+                "A corrupted image in the import set or an installation error may have caused this issue.",
                 ""  # msg
             )
         )
@@ -930,7 +942,9 @@ class Reconstruction(UIGraph):
     displayedAttr2D = makeProperty(QObject, "_displayedAttr2D", displayedAttr2DChanged)   
 
     displayedAttrs3DChanged = Signal()    
-    displayedAttrs3D = Property(QObject, lambda self: self._displayedAttrs3D, notify=displayedAttrs3DChanged)  
+    displayedAttrs3D = Property(QObject, lambda self: self._displayedAttrs3D, notify=displayedAttrs3DChanged)
+    
+    pluginsReloaded = Signal(list)
 
     @Slot(QObject)
     def setActiveNode(self, node, categories=True, inputs=True):
@@ -1153,6 +1167,21 @@ class Reconstruction(UIGraph):
         self._currentViewPath = path
         self.currentViewPathChanged.emit()
 
+    @Slot(str, result="QVariantList")
+    def evaluateMathExpression(self, expr):
+        """ Evaluate a mathematical expression and return the result as a string 
+        Returns a list of 2 values :
+        - the result value
+        - a boolean that indicate if an error occured
+        """
+        mev = MathEvaluator()
+        try:
+            res = mev.evaluate(expr)
+            return [res, False]
+        except Exception as err:
+            self.parent().showMessage(f"Invalid field expression: {expr}", "error")
+            return [None, True]
+
     selectedViewIdChanged = Signal()
     selectedViewId = Property(str, lambda self: self._selectedViewId, setSelectedViewId, notify=selectedViewIdChanged)
     selectedViewpointChanged = Signal()
